import type { APIRoute } from "astro";

export const prerender = false;

const sendEmailViaJMAP = async ({
  subject,
  message,
}: {
  subject: string;
  message: string;
}) => {
  const accessToken = process.env.JMAP_ACCESS_TOKEN
    ? process.env.JMAP_ACCESS_TOKEN
    : import.meta.env.JMAP_ACCESS_TOKEN;
  const accountId = process.env.JMAP_ACCOUNT_ID
    ? process.env.JMAP_ACCOUNT_ID
    : import.meta.env.JMAP_ACCOUNT_ID;
  const fromEmail = process.env.FROM_EMAIL
    ? process.env.FROM_EMAIL
    : import.meta.env.FROM_EMAIL;
  const toEmail = process.env.TO_EMAIL
    ? process.env.TO_EMAIL
    : import.meta.env.TO_EMAIL;
  const apiUrl = "https://api.fastmail.com/jmap/api/";

  if (!accessToken || !accountId || !fromEmail || !toEmail) {
    throw new Error("Missing environment variables configuration");
  }

  const headers = new Headers({
    Authorization: `Bearer ${accessToken}`,
    "Content-Type": "application/json",
    Accept: "application/json",
  });

  try {
    // Get email identities
    const identityRequest = {
      using: ["urn:ietf:params:jmap:core", "urn:ietf:params:jmap:submission"],
      methodCalls: [
        [
          "Identity/get",
          {
            accountId,
            properties: ["id", "email", "name"],
          },
          "0",
        ],
      ],
    };

    const identityResponse = await fetch(apiUrl, {
      method: "POST",
      headers,
      body: JSON.stringify(identityRequest),
    });

    const identityData = await identityResponse.json();
    const identity = identityData.methodResponses[0][1].list.find(
      (id: any) => id.email === fromEmail,
    );

    if (!identity) {
      throw new Error(`No identity found for email: ${fromEmail}`);
    }

    // Get drafts mailbox
    const mailboxRequest = {
      using: ["urn:ietf:params:jmap:core", "urn:ietf:params:jmap:mail"],
      methodCalls: [
        [
          "Mailbox/get",
          {
            accountId,
            properties: ["id", "role"],
          },
          "0",
        ],
      ],
    };

    const mailboxResponse = await fetch(apiUrl, {
      method: "POST",
      headers,
      body: JSON.stringify(mailboxRequest),
    });

    const mailboxData = await mailboxResponse.json();
    const drafts = mailboxData.methodResponses[0][1].list.find(
      (box: any) => box.role === "drafts",
    );

    if (!drafts) {
      throw new Error("Could not find Drafts folder");
    }

    // Create email draft
    const emailRequest = {
      using: [
        "urn:ietf:params:jmap:core",
        "urn:ietf:params:jmap:mail",
        "urn:ietf:params:jmap:submission",
      ],
      methodCalls: [
        [
          "Email/set",
          {
            accountId,
            create: {
              "draft-1": {
                mailboxIds: { [drafts.id]: true },
                from: [{ email: fromEmail, name: identity.name }],
                to: [{ email: toEmail }],
                subject,
                bodyValues: {
                  body1: {
                    value: message,
                    charset: "utf-8",
                  },
                },
                textBody: [
                  {
                    partId: "body1",
                    type: "text/plain",
                  },
                ],
              },
            },
          },
          "0",
        ],
      ],
    };

    const emailResponse = await fetch(apiUrl, {
      method: "POST",
      headers,
      body: JSON.stringify(emailRequest),
    });

    const emailResult = await emailResponse.json();
    const createdId = emailResult.methodResponses[0][1].created["draft-1"].id;

    // Submit email with identity
    const submitRequest = {
      using: [
        "urn:ietf:params:jmap:core",
        "urn:ietf:params:jmap:mail",
        "urn:ietf:params:jmap:submission",
      ],
      methodCalls: [
        [
          "EmailSubmission/set",
          {
            accountId,
            create: {
              "submit-1": {
                emailId: createdId,
                identityId: identity.id,
                envelope: {
                  mailFrom: {
                    email: fromEmail,
                  },
                  rcptTo: [{ email: toEmail }],
                },
              },
            },
          },
          "0",
        ],
      ],
    };

    const submitResponse = await fetch(apiUrl, {
      method: "POST",
      headers,
      body: JSON.stringify(submitRequest),
    });

    const submitResult = await submitResponse.json();

    if (!submitResult.methodResponses[0][1].created?.["submit-1"]) {
      throw new Error(`Submission failed: ${JSON.stringify(submitResult)}`);
    }

    const deleteRequest = {
      using: ["urn:ietf:params:jmap:core", "urn:ietf:params:jmap:mail"],
      methodCalls: [
        [
          "Email/set",
          {
            accountId,
            destroy: [createdId],
          },
          "0",
        ],
      ],
    };

    const deleteResponse = await fetch(apiUrl, {
      method: "POST",
      headers,
      body: JSON.stringify(deleteRequest),
    });

    const deleteResult = await deleteResponse.json();
    if (!deleteResult.methodResponses[0][1].destroyed?.includes(createdId)) {
      console.warn("Failed to delete draft email", deleteResult);
    }

    return { success: true };
  } catch (error) {
    console.error("JMAP Error:", error);
    throw error;
  }
};

export const POST: APIRoute = async ({ request }) => {
  try {
    const data = await request.json();
    const { subject, message } = data;

    if (!subject?.trim() || !message?.trim()) {
      return new Response(
        JSON.stringify({ error: "Subject and message are required" }),
        {
          status: 400,
          headers: { "Content-Type": "application/json" },
        },
      );
    }

    await sendEmailViaJMAP({ subject, message });

    return new Response(
      JSON.stringify({ success: true, message: "Email sent successfully" }),
      {
        status: 200,
        headers: { "Content-Type": "application/json" },
      },
    );
  } catch (error) {
    console.error("API Error:", error);
    return new Response(
      JSON.stringify({
        error: "Failed to send message",
        details: error instanceof Error ? error.message : "Unknown error",
      }),
      {
        status: 500,
        headers: { "Content-Type": "application/json" },
      },
    );
  }
};
